"""
This package contains the basic core protovis resources to be
used by the widgets packaged in the following:

    - tw2.protovis.conventional
    - tw2.protovis.custom
    - tw2.protovis.hierarchies

Get this source from http://github.com/toscawidgets/tw2.protovis.core

Protovis itself can be found here:  http://vis.stanford.edu/protovis/

**Warning**: Protovis itself is deprecated and will be replaced by a new
             javascript library called ``d3``.  You should check out
             ``tw2.d3`` as a replacement.
"""

from widgets import *
